/*
 *   jack-ladspa-host
 *    
 *   Copyright (C) Robert Ham 2002 (node@users.sourceforge.net)
 *    
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __JLH_CHAIN_H__
#define __JLH_CHAIN_H__

#include <glib-2.0/glib.h>
#include <ladspa.h>

#include "process.h"

typedef struct ladspa_holder {
  LADSPA_Handle instance;
  lff_t * control_fifos;
  LADSPA_Data * control_memory;
} ladspa_holder_t;

/** audio_memory indicies */
enum { IN_L = 0,
       IN_R,
       OUT_L,
       OUT_R };

typedef struct ladspa_chain_link {
  gint mono;
  gint enabled;
  
  union {
    struct {
      ladspa_holder_t * left;
      ladspa_holder_t * right;
    } mono_pair;
    
    ladspa_holder_t * stereo;
    
  } holder;
  
  struct plugin * plugin;
  const LADSPA_Descriptor * descriptor;

  /** { left, right } */
  LADSPA_Data * audio_memory[2];
  
  /** { in l, in r, out l, out r } */
  unsigned long * port_indicies;
  
  unsigned long control_port_count;
  unsigned long * control_port_indicies;
  
  struct ladspa_chain_link * next;
  struct ladspa_chain_link * prev;
  
} ladspa_chain_link_t;

/** add a plugin to the end of the chain */
extern void add_plugin (
  process_info_t *,
  ladspa_chain_link_t *);

/** remove a plugin from the chain */
extern ladspa_chain_link_t * remove_plugin (
  process_info_t *,
  gint);

/** enable/disable a plugin */
extern void ablise_plugin (
  process_info_t *,
  gint,
  gint);

/** move a plugin up or down one place in the chain */
extern void move_plugin (
  process_info_t *,
  gint,
  gint);

/** exchange an existing plugin for a newly created one */
extern ladspa_chain_link_t * change_plugin (
  process_info_t *,
  gint,
  ladspa_chain_link_t *);

struct plugin;

extern int init_chain_link (ladspa_chain_link_t * chain_link,
                            struct plugin * plugin);
extern void destroy_chain_link (ladspa_chain_link_t * chain_link);
extern void connect_input_ports (ladspa_chain_link_t * plugin);
extern void connect_output_ports (ladspa_chain_link_t * plugin);
#endif /* __JLH_CHAIN_H__ */
