/*
 *   jack-ladspa-host
 *    
 *   Copyright (C) Robert Ham 2002 (node@users.sourceforge.net)
 *    
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __JLH_GUI_H__
#define __JLH_GUI_H__

#include <gtk/gtk.h>
#include <ladspa.h>

#include "chain.h"
#include "plugin.h"

/** enum for control port types */
typedef enum { JR_CTRL_FLOAT,
            /* JR_CTRL_LOG, */
               JR_CTRL_INT,
               JR_CTRL_BOOL
             } control_type_t;
               

typedef struct port_control {
  GtkWidget * label;
  GtkWidget * control;
  GtkWidget * text;
  lff_t *     control_fifo;
} port_controls_t;

/** the widgets for controlling an ladspa control port.  this is what is
    passed to the callbacks for the widgets, allowing them to send data
    down the fifo, and update the values in the different widgets */
typedef struct port_controls_holder {
  control_type_t type;
  union {
    port_controls_t stereo;
    port_controls_t mono[2];
  } controls;

  /* mono only stuff */
  GtkWidget * lock;
  gint        locked;
  
  gint mono;
  struct plugin_slot * plugin_slot;
  unsigned long port_index;
} port_controls_holder_t;

typedef struct plugin_settings {
  LADSPA_Data * control_values[2];
  gboolean * locks;
  gboolean lock_all;
} plugin_settings_t;

/** a slot in the main gui's box of plugin controls.  this contains all
    the widgets and plugin data for the slot */
typedef struct plugin_slot {
  struct plugin_slot * next;
  struct plugin_slot * prev;
  unsigned long index;
  
  plugin_t * plugin;
  ladspa_chain_link_t * chain_link;
  plugin_settings_t * plugin_settings;
  gint enabled;
  
  /* widgets */
  GtkWidget * main_vbox; /* vbox */
  GtkWidget * top_controls; /* hbox */
  GtkWidget * plugin_selector; /* button */
  GtkWidget * plugin_menu; /* menu */
  GtkWidget * slot_up; /* button */
  GtkWidget * slot_up_image; /* button */
  GtkWidget * slot_remove; /* button */
  GtkWidget * slot_remove_image; /* button */
  GtkWidget * slot_down; /* button */
  GtkWidget * slot_down_image; /* button */
  GtkWidget * enable; /* toggle button */
  GtkWidget * lock_all; /* check button */
  GtkWidget * control_table; /* table for the port controls */
  GtkWidget * separator; /* hseperator */
  
  port_controls_holder_t * port_control_holders; /* port controls array */
  
} plugin_slot_t;


typedef struct guiinfo {
  GtkWidget * main_window;
  GtkWidget * plugin_box;
  GtkWidget * add;
  
  plugin_slot_t * slots;
  plugin_slot_t * slots_end;
  unsigned long slot_count;
  gboolean moving_slot;
  /* whether or not jack has shut us down */
  gboolean shutdown;
  
  lff_t * gui_to_process;
  lff_t * process_to_gui;
  
  plugin_collection_t * plugin_collection;
  process_info_t * process_info;
} guiinfo_t;

extern guiinfo_t global_gui_info;

extern int create_gui (plugin_collection_t * collection, guiinfo_t * guiinfo);
extern void init_plugin_slot (plugin_slot_t * plugin_slot,
                              plugin_collection_t * collection,
                              plugin_t * plugin,
                              ladspa_chain_link_t * chain_link);
extern void add_plugin_slot (guiinfo_t * gui_info,
                             ladspa_chain_link_t * chain_link,
                             plugin_t * plugin);                              
extern void remove_plugin_slot (guiinfo_t * gui_info,
                                plugin_slot_t * plugin_slot);
extern void move_plugin_slot (guiinfo_t * gui_info,
                              plugin_slot_t * plugin_slot,
                              gint up);
void change_plugin_slot (unsigned long plugin_slot_index,
                         ladspa_chain_link_t * chain_link,
                         guiinfo_t * gui_info);
extern void destroy_gui (guiinfo_t * gui_info);
#endif /* __JLH_GUI_H__ */
