/*
 *   jack-ladspa-host
 *    
 *   Copyright (C) Robert Ham 2002 (node@users.sourceforge.net)
 *    
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __JLH_PROCESS_H__
#define __JLH_PROCESS_H__

#include <glib.h>
#include <jack/jack.h>
#include <ladspa.h>

#include "lock-free-fifo.h"


/** this is what gets passed to the process() callback and contains all
    the data the process callback will need */
typedef struct processinfo {

  /** the plugin instance chain */
  struct ladspa_chain_link * chain;
  struct ladspa_chain_link * chain_end;
  
  /** lock free fifo for gui->process() comm */
  lff_t * gui_to_process;
  /** lock free fifo for process()->gui comm */
  lff_t * process_to_gui;
  
  /** jack ports, same arrangement as for the chain link's memory */
  jack_port_t * jack_ports[4];
  
  LADSPA_Data * jack_buffers[4];
  
  jack_client_t * jack_client;
  
} process_info_t;

extern jack_nframes_t sample_rate;
extern jack_nframes_t buffer_size;

extern int process (jack_nframes_t frames, void * data);
extern int init_jack (process_info_t *, const char *);
extern void destroy_process_info (process_info_t *);
#endif /* __JLH_PROCESS_H__ */
